#!/usr/bin/env python

# Super Spelunker 0.0.1
# Copyright (c) 2007 Philip Allison
#
# http://mangobrain.co.uk/
#
# Originally created for the fourth PyWeek Game Contest
# http://www.pyweek.org/4/
#
# Released under the Artistic License - please read LICENSE or License.txt.
# If you did not receive a copy of the license with this file, please contact
# the author via one of the above URLs, stating where you obtained your package.

import os
import sys
import random
random.seed()

class CGlobals:
	pass
globals = CGlobals()

# Package maintainers:
# Change this to the installed location of the 'data' directory
globals.gDatadir = os.path.realpath('data')
# Change this to the installed location of the 'src' directory
sys.path.append(os.path.realpath('src'))

import spMenu

# Define a record for storing our options
class ROptRecord:
	def __init__(self):
		self.setDefaults()
	# Set default options
	def setDefaults(self):
		self.fullscreen = False
		self.sound = True

# Instantiate option class - filled with default options
gOptions = ROptRecord()

# Read in stored options

# Process commandline arguments
# Do this last such that both default and saved options can be overridden
# Always skip the first argument (program name)
skipnext = True
for i in xrange(len(sys.argv)):
	# Skip argument if skip flag is set
	if (skipnext):
		# Clear the flag so we don't just skip all remaining options!
		skipnext = False
		continue
	if ((sys.argv[i] == '--fullscreen') or (sys.argv[i] == '-f')):
		gOptions.fullscreen = (sys.argv[i + 1] == '1')
		skipnext = True
	elif ((sys.argv[i] == '--sound') or (sys.argv[i] == '-s')):
		gOptions.sound = (sys.argv[i + 1] == '1')
		skipnext = True
	else:
		sys.stderr.write('Unrecognised option: ' + sys.argv[i])

#
# Main loop code
#

# Set the maze parameters
globals.gMazeW = 30
globals.gMazeH = 30

# Set the screen parameters
# Screen resolution
globals.gXres = 800
globals.gYres = 600
# Tile width/height
globals.gTileW = 25
globals.gTileH = 25
# HUD height
globals.gHudH = 100
globals.gBossWarning = None
# Game mode
# 0 - Adventure, 1 - Caveflying
globals.gGameMode = 0
# Ship shield units & frames worth of invulnerability on respawn
globals.gMaxShield = 20
globals.gInvulnFrames = 40
# No. of tiles on screen
globals.gRoomW = globals.gXres/globals.gTileW
globals.gRoomH = (globals.gYres-globals.gHudH)/globals.gTileH

# Set up PyGame
import pygame
pygame.display.init()
globals.gScreen = pygame.display.set_mode((globals.gXres, globals.gYres))

# Run main loop in all its glory
globals.gNextLoopClass = spMenu.TitleScreen
globals.gQuit = False
loopargs = None
currentloop = None
# Only allow quit events, for speed
pygame.event.set_allowed(None)
pygame.event.set_allowed(pygame.QUIT)
pygame.event.set_allowed(pygame.KEYDOWN)
pygame.event.clear()
globals.gDirtyRects = []
while (not globals.gQuit):
	# Construct the next main loop class on state change
	del currentloop
	currentloop = globals.gNextLoopClass(globals, loopargs)
	changeloop = False
	loopargs = None
	while ((not globals.gQuit) and (not changeloop)):
		# Run the main loop at 40 FPS
		before = pygame.time.get_ticks()
		pygame.event.pump()
		e = pygame.event.get(pygame.QUIT)
		if (len(e) != 0):
			globals.gQuit = True
		else:
			globals.gKeyState = pygame.key.get_pressed()
			(changeloop, loopargs) = currentloop.tick(globals)
			pygame.time.wait(25 - (pygame.time.get_ticks() - before))
			pygame.display.update(globals.gDirtyRects)
			globals.gDirtyRects = []
